pub struct DataflowGraph {
pub def_id: DefId,
pub span: Span,
pub argc: usize,
pub nodes: GraphNodes,
pub edges: GraphEdges,
pub n_locals: usize,
pub closures: HashSet<DefId>,
}Fields
def_id: DefIdspan: Spanargc: usizenodes: GraphNodesedges: GraphEdgesn_locals: usizeclosures: HashSet<DefId>Implementations
Sourceimpl DataflowGraph
impl DataflowGraph
pub fn to_dot_graph<'tcx>(&self, tcx: &TyCtxt<'tcx>) -> String
Sourceimpl DataflowGraph
impl DataflowGraph
pub fn add_operand( &mut self, operand: &Operand<'_>, dst: Local, block: usize, stmt_idx: usize, )
pub fn parse_place( &mut self, place: &Place<'_>, block: usize, stmt_idx: usize, ) -> Local
pub fn add_statm_to_graph( &mut self, statement: &Statement<'_>, block: usize, stmt_idx: usize, )
pub fn add_terminator_to_graph( &mut self, terminator: &Terminator<'_>, block: usize, stmt_idx: usize, )
pub fn query_node_by_span( &self, span: Span, strict: bool, ) -> Option<(Local, &DataflowNode)>
Sourceimpl DataflowGraph
impl DataflowGraph
pub fn new(def_id: DefId, span: Span, argc: usize, n_locals: usize) -> Self
pub fn node(&self, local: Local) -> &DataflowNode
pub fn node_mut(&mut self, local: Local) -> &mut DataflowNode
pub fn edge(&self, idx: EdgeIdx) -> &DataflowEdge
pub fn is_marker(&self, idx: Local) -> bool
pub fn add_node_edge( &mut self, src: Local, dst: Local, op: EdgeOp, block: usize, statement_index: usize, ) -> EdgeIdx
pub fn add_const_edge( &mut self, src_desc: String, src_ty: String, dst: Local, op: EdgeOp, block: usize, statement_index: usize, ) -> EdgeIdx
pub fn get_upside_idx(&self, node_idx: Local, order: usize) -> Option<Local>
pub fn get_downside_idx(&self, node_idx: Local, order: usize) -> Option<Local>
pub fn is_connected(&self, idx_1: Local, idx_2: Local) -> bool
pub fn param_return_deps(&self) -> IndexVec<Local, bool>
pub fn dfs<F, G>( &self, now: Local, direction: Direction, node_operator: &mut F, edge_validator: &mut G, traverse_all: bool, seen: &mut HashSet<Local>, ) -> (DFSStatus, bool)
pub fn find_first_node<P, E>( &self, start: Local, direction: Direction, node_predicate: &mut P, edge_validator: &mut E, ) -> Option<Local>
pub fn find_all_nodes<P, E>( &self, start: Local, direction: Direction, node_predicate: &mut P, edge_validator: &mut E, ) -> Vec<Local>
pub fn equivalent_edge_validator( graph: &DataflowGraph, idx: EdgeIdx, ) -> DFSStatus
pub fn always_true_edge_validator(_: &DataflowGraph, _: EdgeIdx) -> DFSStatus
pub fn collect_equivalent_locals( &self, local: Local, strict: bool, ) -> HashSet<Local>
fn collect_by_direction( &self, local: Local, self_included: bool, direction: Direction, ) -> HashSet<Local>
pub fn collect_ancestor_locals( &self, local: Local, self_included: bool, ) -> HashSet<Local>
pub fn collect_descending_locals( &self, local: Local, self_included: bool, ) -> HashSet<Local>
pub fn get_field_sequence(&self, local: Local) -> Option<(Local, Vec<usize>)>
Sourcepub fn trace_origin(&self, local: Local) -> Local
pub fn trace_origin(&self, local: Local) -> Local
Follow Copy/Move edges upward to find the root real local behind any copy chains (no projections). Stops at 16 hops to bound cycles.
Sourcepub fn is_from_tuple_field(&self, local: Local) -> bool
pub fn is_from_tuple_field(&self, local: Local) -> bool
Return true when local originates from a tuple field destructuring
(e.g. (tuple.0, tuple.1) after a call returning a tuple).
Follows Copy/Move chains upward through marker nodes and checks for
any Field edge along the projection chain.
Sourcefn marker_to_real(&self, marker: Local) -> Option<Local>
fn marker_to_real(&self, marker: Local) -> Option<Local>
Walk up a projection-marker chain to find the underlying real local.
Sourcefn marker_chain_has_field(&self, marker: Local) -> bool
fn marker_chain_has_field(&self, marker: Local) -> bool
Check whether a projection-marker chain contains a Field edge.